@@ -47,13 +47,6 @@ my %defaultz =
   , ignore => [ qw( / /usr ) ]
 );
 
-# only new directories are used, ignore pre-loads
-# this with unwanted values.
-
-my %found = ();
-
-# saves passing this between import and $handle_args.
-
 my $empty   = q{};
 
 ########################################################################
@@ -68,8 +61,6 @@ my $empty   = q{};
 my $find_dirs
 = sub
 {
-$DB::single = 1;
-
     my $argz    = shift;
     my $base    = basename $argz->{ base };
     my $subdir  
@@ -265,14 +256,26 @@ my $handle_args
     }
 
     local $defaultz{ Bin }
-    = exists $argz{ realbin }
-    ? $FindBin::RealBin
-    : $FindBin::Bin
-    ;
+    = do
+    {
+        if( my $bin = $argz{ Bin } )
+        {
+            -d $bin
+            ? $bin
+            : -f $bin
+            ? dirname $bin
+            : die "Botched Bin: '$bin' neither file nor dir.\n";
+        }
+        else
+        {
+            exists $argz{ realbin }
+            ? $FindBin::RealBin
+            : $FindBin::Bin
+        }
+    };
 
     if( exists $argz{ ignore } )
     {
-$DB::single = 1;
         # ignore is different: it requires a bit of 
         # compilation.
         #
@@ -449,6 +452,14 @@ directories based on $FindBin::Bin.
 
     use FindBin::libs qw( Bin=/tmp base=altlib );
 
+    # if Bin is a file then it's dirname will be used.
+    # useful for ./t files living above files being 
+    # tested. __FILE__ is a token, doesn't work with
+    # qw().
+
+    use FindBin::libs ( 'Bin=' . __FILE__ );
+    use FindBin::libs ( 'Bin=' . __FILE__, 'base=config' );
+
     # skip "use lib", export "@altlib" instead.
 
     use FindBin::libs qw( base=altlib export );
