@@ -1,52 +1,109 @@
-#######################################################################
+########################################################################
 # housekeeping
 ########################################################################
 
-package FindBin::libs   v3.0.0;
-use v5.28;
+package FindBin::libs   v2.12.01;
+use v5.14;
+use strict;
 
 use FindBin;
+
 use File::Basename;
 
 use Carp    qw( croak                   );
-use Cwd     qw( abs_path                );
 use Symbol  qw( qualify qualify_to_ref  );
 
 use File::Spec::Functions
 qw
 (
-    splitpath
-    splitdir
-    catpath
-    catdir
-    canonpath
+    &splitpath
+    &splitdir
+    &catpath
+    &catdir
 );
 
+BEGIN
+{
+    # however... there have been complaints of 
+    # places where abs_path does not work. 
+    #
+    # if abs_path fails on the working directory
+    # then replace it with rel2abs and live with 
+    # possibly slower, redundant directories.
+    #
+    # the abs_path '//' hack allows for testing 
+    # broken abs_path on primitive systems that
+    # cannot handle the rooted system being linked
+    # back to itself.
+
+    use Cwd qw( &abs_path &cwd );
+
+    if
+    (
+        # abs_path has a fixed bug dealing with infinite
+        # recursion. if upping the version of Cwd does 
+        # not fix this then the only other test I can 
+        # think of is ( -e '/.' && -e '/..' && -e '/../.' )
+
+        eval 
+        {
+            abs_path '//';
+            abs_path cwd
+        }
+    )
+    {
+        # abs_path seems clean on this platform.
+    }
+    else
+    {
+        # abs_path seems to be having problems,
+        # fix is to stub it out.
+        #
+        # undef avoids nastygram.
+
+        my $ref = qualify_to_ref 'abs_path', __PACKAGE__;
+
+        my $sub = File::Spec::Functions->can( 'rel2abs' );
+
+        undef &{ $ref };
+
+        *$ref = $sub
+    };
+}
+
 ########################################################################
 # package variables 
 ########################################################################
 
 my %defaultz = 
 (
-    base    => 'lib'
-  , use     => undef
-  , blib    => undef    # prefer ./blib at the first level
+    base    => 'lib',
+    use     => undef,
+    blib    => undef,   # prefer ./blib at the first level
 
-  , subdir  => '',      # add this subdir also if found.
-  , subonly => undef    # leave out lib's, use only subdir.
-  , export  => undef    # push variable into caller's space.
-  , append  => undef    # push onto existing array (vs. overwrite)
-  , verbose => undef    # boolean: print inputs, results.
-  , debug   => undef    # boolean: set internal breakpoints.
+    subdir  => '',      # add this subdir also if found.
+    subonly => undef,   # leave out lib's, use only subdir.
+    export  => undef,   # push variable into caller's space.
+    append  => undef,   # push onto existing array (vs. overwrite)
+    verbose => undef,   # boolean: print inputs, results.
+    debug   => undef,   # boolean: set internal breakpoints.
 
-  , print   => 1        # display the results
+    print   => 1,       # display the results
 
-  , p5lib   => undef    # prefix PERL5LIB with the results
+    p5lib   => undef,   # prefix PERL5LIB with the results
 
-  # dir's to skip looking for ./lib
-  , ignore => [ qw( / /usr ) ]
+    ignore => '/,/usr', # dir's to skip looking for ./lib
 );
 
+# only new directories are used, ignore pre-loads
+# this with unwanted values.
+
+my %found = ();
+
+# saves passing this between import and $handle_args.
+
+my %argz    = ();
+my $verbose = '';
 my $empty   = q{};
 
 ########################################################################
@@ -58,48 +115,14 @@ my $empty   = q{};
 # directory path for it on VMS. Fix is to unshift a leading
 # '' into @dirpath where the leading entry is true.
 
-my $find_dirs
+my $find_libs
 = sub
 {
-    my $argz    = shift;
-    my $base    = basename $argz->{ base };
-    my $subdir  
-    = $argz->{ subdir }
-    ? canonpath $argz->{ subdir }
-    : ''
-    ;
-    my $subonly = $argz->{ subonly } // '';
-    my %seen    = ();
-    my @libz    = ();
+    my $base    = basename ( shift || $argz{ base } );
 
-    my $onepass = $argz->{ export } && $argz->{ scalar };
-    my $ignore  
-    = do
-    {
-        # ignoring nothing may be used with export,
-        # mainly for testing.
+    my $subdir  = $argz{ subdir } || '';
 
-        my $dirz    = $argz->{ ignore };
-        
-        if( @$dirz )
-        {
-            my $altz    
-            = join '|' =>
-            map
-            {
-                catdir $_, $base
-            }
-            @$dirz;
-
-            qr{^ (?: $altz ) $}x
-        }
-        else
-        {
-            qr{^$}
-        }
-    };
-
-    my $verbose = $argz->{ verbose };
+    my $subonly = defined $argz{ subonly };
 
     # for some reason, RH Enterprise V/4 has a 
     # trailing '/'; I havn't seen another copy of 
@@ -113,9 +136,9 @@ my $find_dirs
     # after that splitpath can grab the directory 
     # portion for future use.
 
-    my ( $Bin ) = ( $argz->{ Bin } =~ m{^ (.+) /? $}xs );
+    my ( $Bin ) = ( $argz{ Bin } =~ m{^ (.+) }xs );
 
-    print STDERR "\nSearching $Bin for '$base' ($subdir)...\n"
+    print STDERR "\nSearching $Bin for '$base'...\n"
     if $verbose;
 
     my( $vol, $dir ) = splitpath $Bin, 1;
@@ -128,6 +151,8 @@ my $find_dirs
 
     unshift @dirpath, '' if $dirpath[ 0 ];
 
+    my @libz    = ();
+
     PATH:
     for( 1 .. @dirpath )
     {
@@ -135,53 +160,40 @@ my $find_dirs
         # volume only means something on DOS- & VMS-based
         # filesystems, and adding an empty basename on 
         # *nix is unnecessary.
+        #
+        # HAK ALERT: the poor slobs stuck on windog have an
+        # abs_path that croaks on missing directories. have
+        # to eval the check for subdir's. 
 
-        my $dir = catpath $vol, ( catdir @dirpath, $base ), $empty;
-
-        $dir    =~ $ignore
-        and next;
-
-        -d $dir or next;
-
-        # regardless of how things are symlinked, the abspath
-        # check only puts each directory on the list once.
-
-        my @dirz
-        = do
+        my $abs
+        = eval
         {
-            if( $subdir )
-            {
-                my $sub = catpath $vol, ( catdir $dir, $subdir ), $empty;
+            abs_path
+            catpath $vol, ( catdir @dirpath, $base ), $empty
+        }
+        || '';
 
-                if( -d $sub )
-                {
-                    $subonly
-                    ? ( $sub )
-                    : ( $sub, $dir )
-                }
-                else
-                {
-                    ()
-                }
-            }
-            else
+        my $sub
+        = $subdir
+        ? eval { abs_path ( catpath '', $abs, $subdir ) } || ''
+        : ''
+        ;
+
+        my @search = $subonly ? ( $sub ) : ( $abs, $sub );
+
+        for my $dir ( @search )
+        {
+            if( $dir && -d $dir && ! exists $found{ $dir } )
             {
-                ( $dir )
+                $found{ $dir } = ();
+
+                push @libz, $dir;
+
+                last if $argz{ scalar };
             }
         }
-        or next;
 
-        for my $dir ( @dirz )
-        {
-            $seen{ abs_path $dir } 
-            ||= push @libz, $dir;
-        }
-
-        last if $onepass;
-    }
-    continue
-    {
-        pop @dirpath;
+        pop @dirpath
     }
 
     # caller gets back the existing lib paths 
@@ -209,7 +221,7 @@ my $handle_args
     # anything after the module are options with arguments
     # assigned via '='.
 
-    my %argz
+    %argz
     = map
     {
         my $use_undef
@@ -256,27 +268,23 @@ my $handle_args
     }
 
     local $defaultz{ Bin }
-    = exists $argz{ realbin }
-    ? $FindBin::RealBin
-    : $FindBin::Bin
-    ;
-
-    if( exists $argz{ ignore } )
+    = do
     {
-        # ignore is different: it requires a bit of 
-        # compilation.
-        #
-        # ignore may be empty, mainly for testing.
-
-        if( my $dirz = $argz{ ignore } )
+        if( my $bin = $argz{ Bin } )
         {
-            $argz{ ignore } = [ split ',' => $dirz ];
+            -d $bin
+            ? $bin
+            : -f $bin
+            ? dirname $bin
+            : die "Botched Bin: '$bin' neither file nor dir.\n";
         }
         else
         {
-            $argz{ ignore } = [];
+            exists $argz{ realbin }
+            ? $FindBin::RealBin
+            : $FindBin::Bin
         }
-    }
+    };
 
     # now apply the defaults, then sanity check the result.
     # base is a special case since it always has to exist.
@@ -301,31 +309,50 @@ my $handle_args
     and
     $argz{ export } //= $argz{ base };
 
-    $argz{ verbose } //= '';
+    $argz{ ignore } =
+    [
+        grep { $_ } split /\s*,\s*/, $argz{ ignore }
+    ];
 
-    %argz
+    $verbose = defined $argz{ verbose };
+
+    my $base = $argz{ base };
+
+    # now locate the libraries.
+    #
+    # %found contains the abs_path results for each directory to 
+    # avoid double-including directories.
+    #
+    # note: loop short-curcuts for the (usually) list.
+
+    %found = ();
+
+    for( @{ $argz{ ignore } } )
+    {
+        if( my $dir = eval { abs_path catdir $_, $base } )
+        {
+            if( -d $dir )
+            {
+                $found{ $dir } = 1;
+            }
+        }
+    }
 };
 
 sub import
 {
-    # map used here for -T checks. everything from find_dirs has
-    # already been through abs_path, not much more to check here.
+    &$handle_args;
 
-    my %argz    = &$handle_args;
-    my @libz    
-    = map
-    {
-        my ( $path ) = m{ (.+) }x; 
+    my @libz = $find_libs->();
 
-        ( $path =~ /\S/ )
-        ? $path
-        : ()
-    }
-    $find_dirs->( \%argz )
-    ;
+    # HAK ALERT: the regex does nothing for security,
+    # just dodges -T. putting this down here instead
+    # of inside find_libs allows people to use saner
+    # untainting plans via find_libs.
 
-    my $verbose = $argz{ verbose };
-    my $caller  = caller;
+    @libz   = map { m{ (.+) }xs } @libz;
+
+    my $caller = caller;
 
     if( $verbose || defined $argz{ print } )
     {
@@ -342,7 +369,7 @@ sub import
         # the caller is expecting to exist at runtime -- even
         # if they are empty/undef at the end of it.
 
-        my $ref = qualify_to_ref $argz{ export }, $caller;
+        my $ref     = qualify_to_ref $argz{ export }, $caller;
 
         if( $verbose )
         {
@@ -377,10 +404,10 @@ sub import
         }
     }
 
+    # no 'else', these are not exclusive
+
     if( @libz )
     {
-        # no 'else', these are not exclusive
-
         if( defined $argz{ p5lib } )
         {
             # stuff the lib's found at the front of $ENV{ PERL5LIB }
@@ -397,6 +424,9 @@ sub import
         {
             # this obviously won't work if lib ever depends 
             # on the caller's package.
+            #
+            # it does avoids issues with -T blowing up on the
+            # old eval technique.
 
             require lib;
 
@@ -408,7 +438,9 @@ sub import
 }
 
 # keep require happy
+
 1
+
 __END__
 
 =head1 NAME
@@ -439,6 +471,14 @@ directories based on $FindBin::Bin.
 
     use FindBin::libs qw( Bin=/tmp base=altlib );
 
+    # if Bin is a file then it's dirname will be used.
+    # useful for ./t files living above files being 
+    # tested. __FILE__ is a token, doesn't work with
+    # qw().
+
+    use FindBin::libs ( 'Bin=' . __FILE__ );
+    use FindBin::libs ( 'Bin=' . __FILE__, 'base=config' );
+
     # skip "use lib", export "@altlib" instead.
 
     use FindBin::libs qw( base=altlib export );
@@ -910,7 +950,7 @@ may be limited to *NIX or at least less portable.
 
 PERL5LIB can be used to accomplish the same directory
 lookups as FindBin::libs.  The problem is PERL5LIB often
-contains absolute paths and does not automatically change
+contains absolte paths and does not automatically change
 depending on where tests are run. This can leave you 
 modifying a file, changing directory to see if it works
 with some other code and testing an unmodified version of 
@@ -952,7 +992,7 @@ in the executable path (or cwd) and uses that.
 
 The main problem here is that if the anchor ever changes
 (e.g., when moving code between projects or relocating 
-directories) the path often has
+directories now that SVN supports it) the path often has
 to change in multiple files. The regex also may have to
 support multiple platforms, or be broken into more complicated
 File::Spec code that probably looks pretty much like what
@@ -1017,12 +1057,8 @@ directory paths based on $FindBin::Bin.
 
 =item Older code.
 
-There are versions of the code for Perl from releases
-v5.8, 5.14, and 5.28. Makefile.PL will pick the most recent
-version it can based on $^V at Configure time. As of this
-writing Perl is at 5.32, which would pick the 5.28 release. 
-
-The version-specific code is in ./version/<perl-v-string.
+FindBin::libs_5_8.pm is installed if $^V indicates
+that the running perl is prior to v5.10.
 
 =back
 
@@ -1051,9 +1087,18 @@ bug, this is obviously a somewhat kludgy workaround
 and should be removed (with an added test for a 
 working version) once the File::Spec is fixed.
 
-Note that I have no way to test this on VMS at this
-point so the only thing I can think of is declaring
-the fix a feature.
+=item 
+
+The hack for prior-to-5.12 versions of perl is 
+messy, but is the only I've found that works for
+the moment on *NIX, VMS, and MSW. I am not sure
+whether any of these systems are normally configured
+to share perl modules between versions. If the 
+moduels are not shared on multiple platforms then
+I can make this work by managing the installation
+rather than checking this every time at startup.
+
+For the moment, at least, this seems to work.
 
 =back
 
@@ -1061,6 +1106,19 @@ the fix a feature.
 
 =over 4
 
+=item Module::FromPerlVer
+
+Explains where the installed version comes from.
+
+=item File::Copy::Recursive
+
+Note: Due to issues with File::Copy::Recusive, this is
+using File::Copy::Recursive::Reduced and a version directory
+for the moment. Once the Recursive module has been dealt with
+Module::FromPerlVer shoud Just Work.
+
+=back
+
 =head2 Cwd
 
 A bug in Cwd was fixed with 3.73. That should, hopefully,
@@ -1075,10 +1133,10 @@ Steven Lembark, Workhorse Computing <lembark@wrkhors.com>
 =head1 COPYRIGHT
 
 Copyright (C) 2003-2022, Steven Lembark, Workhorse Computing.
-This code is released under the same terms as Perl-5.28
+This code is released under the same terms as Perl-5.24
 or any later version of Perl.
 
 =head1 LICENSE
 
-This code is released under the same terms as Perl-5.34
+This code is released under the same terms as Perl-5.24
 or any later version of Perl.
